// helpshift.cpp
// Extension lib defines
#define EXTENSION_NAME Helpshift
#define LIB_NAME "Helpshift"
#define MODULE_NAME "helpshift"

// include the Defold SDK
#include <dmsdk/sdk.h>

#include "helpshift.h"

namespace dmHelpshift {

static int Reverse(lua_State* L)
{
    // The number of expected items to be on the Lua stack
    // once this struct goes out of scope
    DM_LUA_STACK_CHECK(L, 1);

    // Check and get parameter string from stack
    char* str = (char*)luaL_checkstring(L, 1);

    // Reverse the string
    int len = strlen(str);
    for(int i = 0; i < len / 2; i++) {
        const char a = str[i];
        const char b = str[len - i - 1];
        str[i] = b;
        str[len - i - 1] = a;
    }

    // Put the reverse string on the stack
    lua_pushstring(L, str);

    // Return 1 item
    return 1;
}

static int Lua_Initialize(lua_State *L)
{
    DM_LUA_STACK_CHECK(L, 0);
    Initialize();
    return 0;
}

static int Lua_ShowConveration(lua_State *L)
{
    DM_LUA_STACK_CHECK(L, 0);
    ShowConveration();
    return 0;
}

// Functions exposed to Lua
static const luaL_reg Module_methods[] =
{
    {"reverse", Reverse},
    {"initialize", Lua_Initialize},
    {"showConveration", Lua_ShowConveration},
    {0, 0}
};

static void LuaInit(lua_State* L)
{
    int top = lua_gettop(L);

    // Register lua names
    luaL_register(L, MODULE_NAME, Module_methods);

    lua_pop(L, 1);
    assert(top == lua_gettop(L));
}

static dmExtension::Result AppInitializeHelpshift(dmExtension::AppParams* params)
{
    dmLogInfo("AppInitializeHelpshift");
    Initialize_Ext();
    dmLogInfo("AppInitializeHelpshift end");
    return dmExtension::RESULT_OK;
}

static dmExtension::Result InitializeHelpshift(dmExtension::Params* params)
{
    // Init Lua
    LuaInit(params->m_L);
    dmLogInfo("Registered %s Extension", MODULE_NAME);
    return dmExtension::RESULT_OK;
}

static dmExtension::Result AppFinalizeHelpshift(dmExtension::AppParams* params)
{
    dmLogInfo("AppFinalizeHelpshift");
    return dmExtension::RESULT_OK;
}

static dmExtension::Result FinalizeHelpshift(dmExtension::Params* params)
{
    dmLogInfo("FinalizeHelpshift");
    return dmExtension::RESULT_OK;
}

static dmExtension::Result OnUpdateHelpshift(dmExtension::Params* params)
{
    dmLogInfo("OnUpdateHelpshift");
    return dmExtension::RESULT_OK;
}

static void OnEventHelpshift(dmExtension::Params* params, const dmExtension::Event* event)
{
    switch(event->m_Event)
    {
        case dmExtension::EVENT_ID_ACTIVATEAPP:
            dmLogInfo("OnEventHelpshift - EVENT_ID_ACTIVATEAPP");
            break;
        case dmExtension::EVENT_ID_DEACTIVATEAPP:
            dmLogInfo("OnEventHelpshift - EVENT_ID_DEACTIVATEAPP");
            break;
        case dmExtension::EVENT_ID_ICONIFYAPP:
            dmLogInfo("OnEventHelpshift - EVENT_ID_ICONIFYAPP");
            break;
        case dmExtension::EVENT_ID_DEICONIFYAPP:
            dmLogInfo("OnEventHelpshift - EVENT_ID_DEICONIFYAPP");
            break;
        default:
            dmLogWarning("OnEventHelpshift - Unknown event id");
            break;
    }
}

}
// Defold SDK uses a macro for setting up extension entry points:
//
// DM_DECLARE_EXTENSION(symbol, name, app_init, app_final, init, update, on_event, final)

// Helpshift is the C++ symbol that holds all relevant extension data.
// It must match the name field in the `ext.manifest`
// DM_DECLARE_EXTENSION(Helpshift, LIB_NAME, AppInitializeHelpshift, AppFinalizeHelpshift, InitializeHelpshift, OnUpdateHelpshift, OnEventHelpshift, FinalizeHelpshift)
DM_DECLARE_EXTENSION(Helpshift, LIB_NAME, dmHelpshift::AppInitializeHelpshift, dmHelpshift::AppFinalizeHelpshift, dmHelpshift::InitializeHelpshift, 0, dmHelpshift::OnEventHelpshift, dmHelpshift::FinalizeHelpshift)
